@@ -5,8 +5,8 @@ from AccessControl.Permission import getPermissions
 from plone import api
 from plone.api.tests.base import INTEGRATION_TESTING
 from plone.app.testing import logout
-from plone.app.testing import TEST_USER_NAME
 from plone.app.testing import TEST_USER_ID
+from plone.app.testing import TEST_USER_NAME
 
 import mock
 import unittest
@@ -328,6 +328,15 @@ class TestPloneApiUser(unittest.TestCase):
         with self.assertRaises(UserNotFoundError):
             api.user.get_roles(username='theurbanspaceman')
 
+    def test_get_roles_anonymous(self):
+        """Test get_roles for an anonymous user."""
+        from AccessControl.users import nobody
+        with api.env.adopt_user(user=nobody):
+            self.assertEqual(
+                api.user.get_roles(),
+                ('Anonymous', )
+            )
+
     def test_get_roles_in_context(self):
         """Test get local and inherited roles for a user on an object"""
         api.user.create(
@@ -638,7 +647,7 @@ class TestPloneApiUser(unittest.TestCase):
         with self.assertRaises(MissingParameterError):
             api.user.revoke_roles()
 
-    @unittest.skip("Getting the Anonymous user does not work like this.")
+    @unittest.skip('Getting the Anonymous user does not work like this.')
     def test_revoke_roles_from_anonymous(self):
         """Test revoking roles from an Anonymous user."""
         api.user.revoke_roles(username='Anonymous User', roles=['Reviewer'])
